// File: Mutant.cpp
// Author: Bob Rubbens - Knights of the Compiler
// Creation date: 2015-03-22
// Contact: http://plusminos.nl - @broervanlisa - gmail (bobrubbens)

// Public
#include <vector>

// Private
#include "game/Mutant.hpp"
#include "main/global.hpp"
#include "main/random.hpp" 

float MUTANT_SPEED = 100;
bool MUTANT_TOME = false;
bool MUTANT_TOLIMB = true;
int MUTANT_SEEKING = 0;
int MUTANT_RESPAWNING = 1;
int MUTANT_DURATION = 3000;
int MUTANT_LIVES = 2;

SDL_Point generateRandomPositionOutside() {
	switch (rand(4)) {
	case 0:
		return {rand(SCR_W + 200), -100};
	case 1:
		return {SCR_W + 100, rand(SCR_H + 200)};
	case 2:
		return {rand(SCR_W + 200), SCR_H + 100};
	case 3:
		return {-100, rand(SCR_H + 200)};
	default:
		return {0, 0};
	}
}

Mutant::Mutant() {
	sprite = g_tm->getTexture("game", "alien.png");

	SDL_Rect srcrect;
	srcrect.x = rand(SCR_W - 40);
	srcrect.y = rand(SCR_H - 40);
	srcrect.w = 40;
	srcrect.h = 40;
	sprite.setSrcRect(srcrect);
	sprite.setSize(40, 40);
	sprite.setOrigin(20, 20);

	auto p = generateRandomPositionOutside();
	sprite.setPosition(p.x, p.y);
	x = p.x;
	y = p.y;
}

Mutant::~Mutant() {

}

void Mutant::logic(float delta, Player *player, std::vector<Seed>& seeds) {
	if (blinking) {
		if (SDL_GetTicks() - blinkStart >= 3000) {
			blinking = false;
			sprite.setColor(255, 255, 255);
		} else {
			float d = (SDL_GetTicks() - blinkStart) / 3000.0;
			sprite.setColor(255, 255 * d, 255 * d);
		}
	}

	if (player->getState() == PLAYER_MOVING) {
		hasCollided = false;
	}

	if (state == MUTANT_SEEKING) {
		int tx, ty;
		if (target == MUTANT_TOME) { // Me
			tx = player->getMeX();
			ty = player->getMeY();
		} else { // Limb
			tx = player->getLimbX();
			ty = player->getLimbY();
		}

		float dx = tx - x;
		float dy = ty - y;
		float len = std::sqrt(dx * dx + dy * dy);
		dx /= (float) len;
		dy /= (float) len;

		x += dx * MUTANT_SPEED * delta;
		y += dy * MUTANT_SPEED * delta;

		sprite.setPosition(x, y);

		SDL_Rect mutantRect = getBounds();
		SDL_Rect meBounds = player->getMeBounds();
		SDL_Rect limbBounds = player->getLimbBounds();
		SDL_Rect is1, is2;
		SDL_IntersectRect(&mutantRect, &meBounds, &is1);
		SDL_IntersectRect(&mutantRect, &limbBounds, &is2);

		if (!SDL_RectEmpty(&is1) || !SDL_RectEmpty(&is2)) {
			if (player->getState() != PLAYER_SWITCHING) {
				state = MUTANT_RESPAWNING;
				stateStart = SDL_GetTicks();
				player->damage();
			} else {
				if (!blinking) {
					hp--;
					hasCollided = true;
					blinking = true;
					blinkStart = SDL_GetTicks();
					if (hp == 0) {
						state = MUTANT_RESPAWNING;
						stateStart = SDL_GetTicks();
					}
				} else {
					if (!hasCollided) {
						hasCollided = true;
						player->damage();
					}
				}
			}
		} else {
			for (auto it = seeds.begin(); it != seeds.end();) {
				SDL_Rect sr = it->getBounds();
				if (SDL_HasIntersection(&sr, &mutantRect)) {
					it = seeds.erase(it);
				} else {
					it++;
				}
			}
		}
	} else if (state == MUTANT_RESPAWNING) {
		float dt = (SDL_GetTicks() - stateStart) / (float) MUTANT_DURATION;
		if (dt < 1) {
			sprite.setSize(dt * 100 + 40, dt * 100 + 40);	
			sprite.setAlpha((1 - dt) * 255);
			sprite.setOrigin(sprite.getRenderWidth() / 2, sprite.getRenderHeight() / 2);
		} else {
			state = MUTANT_SEEKING;
			
			SDL_Rect srcrect;
			srcrect.x = rand(SCR_W - 40);
			srcrect.y = rand(SCR_H - 40);
			srcrect.w = 40;
			srcrect.h = 40;
			sprite.setSrcRect(srcrect);

			sprite.setAlpha(255);
			sprite.setSize(40, 40);
			sprite.setOrigin(20, 20);

			auto p = generateRandomPositionOutside();
			x = p.x;
			y = p.y;
			sprite.setPosition(x, y);
			hp = MUTANT_LIVES;

			blinking = false;
			blinkStart = 0;
			sprite.setColor(255, 255, 255);
		}
	}
}

void Mutant::render() const {
	sprite.render();
}

void Mutant::followMe() {
	target = MUTANT_TOME;
}

void Mutant::followLimb() {
	target = MUTANT_TOLIMB;
}

SDL_Rect Mutant::getBounds() const {
	SDL_Rect mutantRect;
	mutantRect.w = sprite.getRenderWidth();
	mutantRect.h = sprite.getRenderHeight();
	mutantRect.x = x - mutantRect.w / 2;
	mutantRect.y = y - mutantRect.h / 2;
	
	return mutantRect;
}
