// File: Player.cpp
// Author: Bob Rubbens - Knights of the Compiler
// Creation date: 2015-03-21
// Contact: http://plusminos.nl - @broervanlisa - gmail (bobrubbens)

// Public

// Private
#include "game/Player.hpp"
#include "main/global.hpp"

float LIMB_SPEED = 400;
int PLAYER_SWITCHTIME =  500;
int const PLAYER_MOVING = 0;
int const PLAYER_SWITCHING = 1;
int const PLAYER_SIZE = 20;
float SMOOTH_ME = 0.4;
float SMOOTH_LIMB = 0.6;
float SMOOTH_LIMB_OWN = 0.2;
float PLAYER_LIVES = 5;
int BLINK_DURATION = 3000;

Player::Player(bool menumode_) : menumode{menumode_} {
	meSprite = g_tm->getTexture("game", "me.png");
	meSprite.setOrigin(meSprite.getRenderWidth() / 2, meSprite.getRenderHeight() / 2);

	limbSprite = g_tm->getTexture("game", "limb.png");
	limbSprite.setOrigin(limbSprite.getRenderWidth() / 2, limbSprite.getRenderHeight() / 2);

	SDL_GetRelativeMouseState(&currMx, &currMy);

	for (int i = 0; i < PLAYER_LIVES; i++) {
		hearts.push_back(g_tm->getTexture("game", "heart.png"));
		hearts.back().setPosition(10 + i * 100 + i * 10, 10);
	}

	lives = PLAYER_LIVES;

	if (menumode) {
		meX = 50;
		meY = 200;
		limbX = SCR_W - 30 - 30;
		currMx = 30;
		currMy = 200;
	}
}

Player::~Player() {

}

// TODO: Muis voor menu mode
void Player::event(SDL_Event& e) {
	if (e.type == SDL_MOUSEMOTION) {
		if (!menumode) currMx += e.motion.xrel;
		currMy += e.motion.yrel;
	} else if (e.type == SDL_KEYDOWN && e.key.repeat == 0) {
		if (e.key.keysym.sym == SDLK_w) {
			if (!menumode) dy--;
		} else if (e.key.keysym.sym == SDLK_a) {
			if (!menumode) dx--;
		} else if (e.key.keysym.sym == SDLK_s) {
			if (!menumode) dy++;
		} else if (e.key.keysym.sym == SDLK_d) {
			if (!menumode) dx++;
		}
		
		if (state == PLAYER_MOVING && e.key.keysym.sym == SDLK_SPACE) {
			state = PLAYER_SWITCHING;
			stateStart = SDL_GetTicks();
			
			limbToX = meX;
			limbToY = meY;
			meToX = meX + limbX;
			meToY = meY + limbY;
		}
	} else if (e.type == SDL_KEYUP) {
		if (e.key.keysym.sym == SDLK_w) {
			if (!menumode) dy++;
		} else if (e.key.keysym.sym == SDLK_a) {
			if (!menumode) dx++;
		} else if (e.key.keysym.sym == SDLK_s) {
			if (!menumode) dy--;
		} else if (e.key.keysym.sym == SDLK_d) {
			if (!menumode) dx--;
		}
	}
}

void Player::logic(float delta) {
	if (blinking) {
		float dt = (SDL_GetTicks() - blinkStart) / (float) BLINK_DURATION;
		if (dt < 1) {
			meSprite.setColor(255, 255 * dt, 255 * dt);
			limbSprite.setColor(255, 255 * dt, 255 * dt);
		} else {
			blinking = false;
			meSprite.setColor(255, 255, 255);
			limbSprite.setColor(255, 255, 255);
		}
	}

	if (state == PLAYER_MOVING) {
		meX = currMx + mouseOffX;
		meY = currMy + mouseOffY;
		meSprite.setPosition(
				SMOOTH_ME * meSprite.getX() + (1 - SMOOTH_ME) * meX,
				SMOOTH_ME * meSprite.getY() + (1 - SMOOTH_ME) * meY);
		if (meX < 0) meX = 0;
		if (meX >= SCR_W) meX = SCR_W;
		if (meY < 0) meY = 0;
		if (meY >= SCR_H) meY = SCR_H;

		if (meX == 0) {
			currMx = 0;
			mouseOffX = 0;
		}
		if (meY == 0) {
			currMy = 0;
			mouseOffY = 0;
		}
		if (meX == SCR_W) {
			currMx = SCR_W;
			mouseOffX = 0;
		}
		if (meY == SCR_H) {
			currMy = SCR_H;
			mouseOffY = 0;
		}

		float len = std::sqrt(dx * dx + dy * dy);
		if (len > 0) {
			limbX = SMOOTH_LIMB_OWN * limbX + (1 - SMOOTH_LIMB_OWN) * (limbX + (dx/len) * delta * LIMB_SPEED);
			limbY = SMOOTH_LIMB_OWN * limbY + (1 - SMOOTH_LIMB_OWN) * (limbY + (dy/len) * delta * LIMB_SPEED);
		}

		if (meX + limbX < 0) {
			limbX += (meX + limbX) * -1;
		}
		if (meX + limbX > SCR_W) {
			limbX -= ((meX + limbX) - SCR_W);
		}
		if (meY + limbY < 0) {
			limbY += (meY + limbY) * -1;
		}
		if (meY + limbY > SCR_H) {
			limbY -= ((meY + limbY) - SCR_H);
		}

		// Smooth that shit
		limbSprite.setPosition(
				limbSprite.getX() * SMOOTH_LIMB + (meX + limbX) * (1 - SMOOTH_LIMB),
				limbSprite.getY() * SMOOTH_LIMB + (meY + limbY) * (1 - SMOOTH_LIMB));
	} else if (state == PLAYER_SWITCHING) {
		float dt = (SDL_GetTicks() - stateStart) / (float) PLAYER_SWITCHTIME;

		if (dt >= 1) {
			meX = meToX;
			meY = meToY;
			limbX = limbToX - meToX;
			limbY = limbToY - meToY;
			state = PLAYER_MOVING;

			mouseOffX = meX - currMx;
			mouseOffY = meY - currMy;

			meSprite.setPosition(meX, meY);
			limbSprite.setPosition(meX + limbX, meY + limbY);
		} else {
			dt = 6 * dt * dt * dt * dt * dt - 15 * dt * dt * dt * dt + 10 * dt * dt * dt;
			meX = limbToX * (1 - dt) + meToX * dt;
			meY = limbToY * (1 - dt) + meToY * dt;
			limbX = meToX * (1 - dt) + limbToX * dt;
			limbY = meToY * (1 - dt) + limbToY * dt;

			meSprite.setPosition(meX, meY);
			limbSprite.setPosition(limbX, limbY);
		}
	}
}

void Player::render() const {
	meSprite.render();
	limbSprite.render();
	for (int i = 0; i < lives; i++) {
		if (!menumode) hearts[i].render();
	}
}

int Player::getLimbX() {
	if (state == PLAYER_MOVING) {
		return meX + limbX;
	} else if (state == PLAYER_SWITCHING) {
		return limbX;
	}

	NNB_ERROR << "Undefined state";

	return -1;
}

int Player::getLimbY() {
	if (state == PLAYER_MOVING) {
		return meY + limbY;
	} else if (state == PLAYER_SWITCHING) {
		return limbY;
	}

	NNB_ERROR << "Undefined state";

	return -1;
}

int Player::getMeX() {
	return meX;
}

int Player::getMeY() {
	return meY;
}

int Player::getState() {
	return state;
}

SDL_Rect Player::getMeBounds() {
	SDL_Rect r;
	r.w = PLAYER_SIZE;
	r.h = PLAYER_SIZE;
	r.x = meX - r.w / 2;
	r.y = meY - r.h / 2;
	
	return r;
}

SDL_Rect Player::getLimbBounds() {
	SDL_Rect r;
	r.w = PLAYER_SIZE;
	r.h = PLAYER_SIZE;
	r.x = getLimbX() - r.w / 2;
	r.y = getLimbY() - r.h / 2;

	return r;
}

void Player::damage() {
	if (!blinking) {
		lives--;
		blinking = true;
		blinkStart = SDL_GetTicks();
	}
}

int Player::getLives() {
	return lives;
}
