// File: Seed.cpp
// Author: Bob Rubbens - Knights of the Compiler
// Creation date: 2015-03-22
// Contact: http://plusminos.nl - @broervanlisa - gmail (bobrubbens)

// Public
#include <cmath>

// Private
#include "game/Seed.hpp"
#include "main/global.hpp"
#include "nnb/utils/constants.hpp"
#include "main/random.hpp"

int const SEED_OK = 0;
int const SEED_REMOVE = 1;
int const SEED_SPAWN = 2;

Seed::Seed(int x_, int y_) :
x{(float) x_},
y{(float) y_},
direction{(double) rand(360)},
walkStartX{x_},
walkStartY{y_},
walkEndX{x_},
walkEndY{y_},
walkStart{0},
walkDuration{0} {
	sprite = g_tm->getTexture("game", "seed.png");
	sprite.setOrigin(sprite.getRenderWidth() / 2, sprite.getRenderHeight() / 2);
	sprite.setRotationCenter({sprite.getRenderWidth() / 2, sprite.getRenderHeight() / 2});
}

Seed::~Seed() {

}

void Seed::event(SDL_Event& e) {

}

int Seed::logic(float delta, Player *player) {
	//sprite.setRotation(sprite.getRotation() + 45 * delta);
	if (SDL_GetTicks() - rotStart > rotDuration) {
		rotDir = rand(2) == 0 ? 1 : -1;
		rotDuration = rand(1000) + 1000;
		rotStart = SDL_GetTicks();
		rotSpd = 30 + rand(150);
	} 
	sprite.setRotation(sprite.getRotation() + rotSpd * rotDir * delta);

	if (SDL_GetTicks() - walkStart > walkDuration) {
		walkStartX = walkEndX;
		walkStartY = walkEndY;
		walkEndX = 30 + rand(SCR_W - 60);
		walkEndY = 30 + rand(SCR_H - 60);

		walkStart = SDL_GetTicks();
		walkSpd = 20 + rand(80);
		walkDuration = std::sqrt(std::pow(walkEndX - walkStartX, 2) + std::pow(walkEndY - walkStartY, 2)) / walkSpd * 1000;
	}
	
	float dt = (SDL_GetTicks() - walkStart) / (float) walkDuration;
	x = walkEndX * dt + walkStartX * (1 - dt);
	y = walkEndY * dt + walkStartY * (1 - dt);
	if (x < 30) x = 30;
	if (y < 30) y = 30;
	if (x > SCR_W - 30) x = SCR_W - 30;
	if (y > SCR_H - 30) y = SCR_H - 30;
	sprite.setPosition(x, y);

	if (player->getState() == PLAYER_SWITCHING) {
		SDL_Rect seedBounds;
		SDL_Rect meBounds;
		SDL_Rect limbBounds;
		SDL_Rect is1, is2;

		seedBounds.w = sprite.getRenderWidth();
		seedBounds.h = sprite.getRenderHeight();
		seedBounds.x = x - seedBounds.w / 2;
		seedBounds.y = y - seedBounds.h / 2;

		meBounds.x = player->getMeX() - PLAYER_SIZE / 2;
		meBounds.y = player->getMeY() - PLAYER_SIZE / 2;
		meBounds.w = PLAYER_SIZE;
		meBounds.h = PLAYER_SIZE;

		limbBounds.x = player->getLimbX() - PLAYER_SIZE / 2;
		limbBounds.y = player->getLimbY() - PLAYER_SIZE / 2;
		limbBounds.w = PLAYER_SIZE;
		limbBounds.h = PLAYER_SIZE;

		SDL_IntersectRect(&seedBounds, &meBounds, &is1);
		SDL_IntersectRect(&seedBounds, &limbBounds, &is2);

		if (!SDL_RectEmpty(&is1) || !SDL_RectEmpty(&is2)) {
			return SEED_REMOVE;
		}
	}

	// TODO: Timing

	return SEED_OK;
}

void Seed::render() const {
	sprite.render();
}

SDL_Rect Seed::getBounds() const {
	SDL_Rect r;
	r.w = sprite.getRenderWidth();
	r.h = sprite.getRenderHeight();
	r.x = x - r.w / 2;
	r.y = y - r.h / 2;

	return r;
}
