// File: Target.cpp
// Author: Bob Rubbens - Knights of the Compiler
// Creation date: 2015-03-22
// Contact: http://plusminos.nl - @broervanlisa - gmail (bobrubbens)

// Public

// Private
#include "game/Target.hpp"
#include "main/global.hpp"

int TARGET_OK = 0;
int TARGET_REMOVE = 1;
int TARGET_STARTSIZE = 140;
float TARGET_SMOOTH = 0.9;

Target::Target(int x_, int y_) :
x{x_},
y{y_} {
	sprite = g_tm->getTexture("game", "target.png");
	sprite.setSize(TARGET_STARTSIZE, TARGET_STARTSIZE);
	sprite.setOrigin(sprite.getRenderWidth() / 2, sprite.getRenderHeight() / 2);
	sprite.setPosition(x, y);
}

Target::~Target() {

}

void Target::event(SDL_Event& e) {

}

int Target::logic(float delta, Player *player) {
	SDL_Rect bounds;
	bounds.w = health / 100 * TARGET_STARTSIZE;
	bounds.h = health / 100 * TARGET_STARTSIZE;
	bounds.x = x - bounds.w / 2;
	bounds.y = y - bounds.h / 2;

	SDL_Rect meBounds;
	meBounds.w = PLAYER_SIZE;
	meBounds.h = PLAYER_SIZE;
	meBounds.x = player->getMeX() - PLAYER_SIZE / 2;
	meBounds.y = player->getMeY() - PLAYER_SIZE / 2;

	SDL_Rect limbBounds;
	limbBounds.w = PLAYER_SIZE;
	limbBounds.h = PLAYER_SIZE;
	limbBounds.x = player->getLimbX() - PLAYER_SIZE / 2;
	limbBounds.y = player->getLimbY() - PLAYER_SIZE / 2;

	bool collision = false;
	SDL_Rect intersection;
	SDL_IntersectRect(&bounds, &meBounds, &intersection);

	if (!SDL_RectEmpty(&intersection)) {
		collision = true;
	}

	SDL_IntersectRect(&bounds, &limbBounds, &intersection);
	if (!SDL_RectEmpty(&intersection)) {
		collision = true;
	}

	if (collision) {
		health -= 30 * delta;
	}

	actualHealth = TARGET_SMOOTH * actualHealth + (1 - TARGET_SMOOTH) * health;

	sprite.setSize(TARGET_STARTSIZE * actualHealth / 100, TARGET_STARTSIZE * actualHealth / 100);
	sprite.setOrigin(sprite.getRenderWidth() / 2, sprite.getRenderHeight() / 2);

	if (health <= 10) {
		return TARGET_REMOVE;
	}

	return TARGET_OK;

}

void Target::render() const {
	sprite.render();
}

int Target::getX() const {
	return x;
}

int Target::getY() const {
	return y;
}
