// File: gsGame.cpp
// Author: Bob Rubbens - Knights of the Compiler
// Creation date: 2014-11-07
// Contact: http://plusminos.nl - @broervanlisa - gmail (bobrubbens)

// Public
#include <cmath>
#include <SDL2/SDL.h>
#include <string>
#include <sstream>

// Private
#include "gamestates/gsGame.hpp"
#include "main/global.hpp"
#include "nnb/log/log.hpp"
#include "nnb/states/GameState.hpp"
#include "nnb/utils/unique_ptr.hpp"
#include "main/random.hpp"

gsGame::gsGame() {
	spawnTarget();
	spawnTarget();
	spawnTarget();

	scoreText = g_fm->getText("04b03.ttf", 100);
	scoreText.setVAlign(nnb::VAlign::BOTTOM);
	scoreText.setHAlign(nnb::HAlign::RIGHT);
	scoreText.setPosition(680, 610);
	scoreText.setColor(gb_4);
	scorePicture = g_tm->getTexture("game", "seed.png");
	scorePicture.setSize(100, 100);
	scorePicture.setOrigin(50, 50);
	scorePicture.setPosition(740, 540);
}

gsGame::~gsGame() {
}

void gsGame::event(SDL_Event& e) {
	player.event(e);

	switch (e.type) {
		case SDL_QUIT:
			g_gm->exit();
			break;
		case SDL_MOUSEBUTTONUP:
			mx = e.button.x;
			my = e.button.y;
			break;
		case SDL_KEYDOWN:
			if (e.key.keysym.sym == SDLK_ESCAPE) {
				//g_gm->exit();
				g_gm->setState("main");
			}
		default:
			break;
	}
}

void gsGame::logic(float delta) {
	// No logic here
	
	// Just madness
	
	if (score >= spawnMutantAt) {
		mutants.emplace_back();
		if (followWho) {
			mutants.back().followMe();
		} else {
			mutants.back().followLimb();
		}
		followWho = !followWho;
		spawnMutantAt += 10;

		if (spawnMutantAt == 50) {
			spawnMutantAt = 999999;
		}
	}
	
	player.logic(delta);

	int spawn = 0;
	for (auto i = targets.begin(); i != targets.end();) {
		if (i->logic(delta, &player) == TARGET_REMOVE) {
			int tx = i->getX();
			int ty = i->getY();

			int seedAmount = rand(5) + 2;
			for (int j = 0; j < seedAmount; ++j) {
				seeds.emplace_back(tx, ty);
			}

			i = targets.erase(i);
			spawn++;
			if (targets.size() > 1 && rand(targets.size()) == 0) {
				spawn++;
			}
		} else {
			i++;
		}
	}

	for (auto i = seeds.begin(); i != seeds.end();) {
		if (i->logic(delta, &player) == SEED_REMOVE) {
			i = seeds.erase(i);
			score += 1;
		} else {
			i++;
		}
	}
	std::stringstream ss;
	ss << score << "x";
	scoreText.setText(ss.str());

	for (int i = 0; i < spawn; ++i) {
	   spawnTarget();
	}

	for (auto& mutant : mutants) {
		mutant.logic(delta, &player, seeds);
	}

	if (player.getLives() == 0) {
		finalScore = score;
		g_gm->setState("gameover");
	}
}

void gsGame::render() const {
	SDL_SetRenderDrawColor(g_renderer, gb_1.r, gb_1.g, gb_1.b, 255);
	SDL_RenderClear(g_renderer);

	for (auto& target : targets) {
		target.render();
	}

	for (auto& seed : seeds) {
		seed.render();
	}

	for (auto& mutant : mutants) {
		mutant.render();
	}

	player.render();

	scoreText.render();
	scorePicture.render();

	SDL_RenderPresent(g_renderer);
}

void gsGame::spawnTarget() {
	int x = 100 + rand(SCR_W - 200);
	int y = 100 + rand(SCR_H - 200);

	targets.emplace_back(x, y);
}
