// Author: Bob Rubbens - Knights of the Compiler
// Creation date: 2014-11-07
// Contact: http://plusminos.nl - @broervanlisa - gmail (bobrubbens)

// Public
#include <cmath>
#include <iostream>
#include <SDL2/SDL.h>
#include <string>
#include <vector>

// Private
#include "gamestates/gsMainMenu.hpp"
#include "main/global.hpp"
#include "nnb/log/log.hpp"
#include "nnb/utils/constants.hpp"
#include "nnb/utils/SDL_Helpers.hpp"

gsMainMenu::gsMainMenu() : player(true) {
	title = g_fm->getText("04b03.ttf", 30);
	title.setText("No one can stop");
	title.setColor(gb_4);
	title.setHAlign(nnb::HAlign::CENTER);
	title.setVAlign(nnb::VAlign::TOP);
	title.setPosition(SCR_W / 2, 40);

	title2 = g_fm->getText("04b03.ttf", 70);
	title2.setText("THE MUTANT TERROR");
	title2.setColor(gb_4);
	title2.setHAlign(nnb::HAlign::CENTER);
	title2.setPosition(SCR_W / 2, 70);

	subtitle = g_fm->getText("04b03.ttf", 30);
	subtitle.setText("A gaem by bobismijnnaam");
	subtitle.setColor(gb_4);
	subtitle.setHAlign(nnb::HAlign::CENTER);
	subtitle.setPosition(SCR_W / 2, 140);

	play = g_fm->getText("04b03.ttf", 50);
	play.setText("Play");
	play.setColor(gb_4);
	play.setHAlign(nnb::HAlign::CENTER);
	play.setPosition(SCR_W / 2, 200);

	tutorial = g_fm->getText("04b03.ttf", 50);
	tutorial.setText("Tutorial");
	tutorial.setColor(gb_4);
	tutorial.setHAlign(nnb::HAlign::CENTER);
	tutorial.setPosition(SCR_W / 2, 300);

	highscores = g_fm->getText("04b03.ttf", 50);
	highscores.setText("Highscores");
	highscores.setColor(gb_4);
	highscores.setHAlign(nnb::HAlign::CENTER);
	highscores.setPosition(SCR_W / 2, 400);

	goodbye = g_fm->getText("04b03.ttf", 50);
	goodbye.setText("Exit (or press escape)");
	goodbye.setColor(gb_4);
	goodbye.setHAlign(nnb::HAlign::CENTER);
	goodbye.setPosition(SCR_W / 2, 500);

}

gsMainMenu::~gsMainMenu() {
}

void gsMainMenu::event(SDL_Event& e) {
	player.event(e);

	switch (e.type) {
		case SDL_QUIT:
			g_gm->exit();
			NNB_LOG << "Exit was requested";
			break;
		case SDL_MOUSEBUTTONUP:
			NNB_LOG << "Mouse button was pressed at (" << e.button.x << "; " << e.button.y << ")";
			break;
		case SDL_KEYDOWN:
			if (e.key.keysym.sym == SDLK_ESCAPE) {
				g_gm->exit();
			}
		default:
			break;
	}

}

void gsMainMenu::logic(float delta) {
	// No logic here
	
	// Haha, jk
	
	player.logic(delta);

	SDL_Rect playBB = play.getBounds();
	SDL_Rect tutBB = tutorial.getBounds();
	SDL_Rect highBB = highscores.getBounds();
	SDL_Rect exitBB = goodbye.getBounds();
	SDL_Rect meBB = player.getMeBounds();

	if (SDL_HasIntersection(&playBB, &meBB)) {
		changeStart = SDL_GetTicks();
		target = 0;
	} else if (SDL_HasIntersection(&tutBB, &meBB)) {
		changeStart = SDL_GetTicks();
		target = 1;
	} else if (SDL_HasIntersection(&highBB, &meBB)) {
		changeStart = SDL_GetTicks();
		target = 2;
	} else if (SDL_HasIntersection(&exitBB, &meBB)) {
		changeStart = SDL_GetTicks();
		target = 3;
	}

	if (target != -1) {
		float d = (SDL_GetTicks() - changeStart) / (float) 800;
		switch (target) {
		case 0:
			play.setColor((1 - d) * 255 + d * gb_4.r, (1 - d)*255 + d * gb_4.g, (1 - d)*255 + d * gb_4.b);
			break;
		case 1:
			tutorial.setColor((1 - d) * 255 + d * gb_4.r, (1 - d)*255 + d * gb_4.g, (1 - d)*255 + d * gb_4.b);
			break;
		case 2:
			highscores.setColor((1 - d) * 255 + d * gb_4.r, (1 - d)*255 + d * gb_4.g, (1 - d)*255 + d * gb_4.b);
			break;
		case 3:
			goodbye.setColor((1 - d) * 255 + d * gb_4.r, (1 - d)*255 + d * gb_4.g, (1 - d)*255 + d * gb_4.b);
			break;
		}
	}

	if (target != -1 && SDL_GetTicks() - changeStart > 800) {
		switch (target) {
		case 0:
			g_gm->setState("game");
			break;
		case 1:
			g_gm->setState("tutorial");
			break;
		case 2:
			g_gm->setState("highscores");
			break;
		case 3:
			g_gm->exit();
			break;
		}
	}
}

void gsMainMenu::render() const {
	SDL_SetRenderDrawColor(g_renderer, gb_1.r, gb_1.g, gb_1.b, 255);
	SDL_RenderClear(g_renderer);

	title.render();

	title2.render();

	subtitle.render();

	play.render();

	tutorial.render();

	highscores.render();

	goodbye.render();

	player.render();

	SDL_RenderPresent(g_renderer);
}
